%Amanda Howard
%Division of Applied Mathematics, Brown University
%April 2018

%This Matlab code generates the two-body resistance functions from the FCM
%mobility matrix. The output is written in a text file in three vectors,
%designed to be copied into C++ code. 

%The FCM resistance functions are given in the form
%R = c_0 + c_1 \epsilon + c_2 \epsilon^2 + c_3 \epsilon^3 + c_4 \epsilon^4
%The five coeffiencts c_i are given for each resistance function. 

%FCM_COEFF_ABC has the coefficients for the FCM terms for XA, YA, YB, XC, YC
%It is of the form: XA_11, XA_12, XA_22, XA_21, YA_11, YA_12, YA_22,
%YA_21, YB_11, YB_12, YB_22, YB_21, ...
%Each term has 5 coefficients, for a total of 5*4*5 = 100 entries for each
%lambda.
%The coefficients are stored in the form: 
%XA_11 = FCM_COEFF_ABC[0] + FCM_COEFF_ABC[1]*epsilon +
    %FCM_COEFF_ABC[2]*epsilon^2 + FCM_COEFF_ABC[3]*epsilon^3 +
    %FCM_COEFF_ABC[4]*epsilon^4
%XA_12 = FCM_COEFF_ABC[5] + FCM_COEFF_ABC[6]*epsilon +
    %FCM_COEFF_ABC[7]*epsilon^2 + FCM_COEFF_ABC[8]*epsilon^3 +
    %FCM_COEFF_ABC[9]*epsilon^4
%XA_22 = FCM_COEFF_ABC[10] + FCM_COEFF_ABC[11]*epsilon +
    %FCM_COEFF_ABC[12]*epsilon^2 + FCM_COEFF_ABC[13]*epsilon^3 +
    %FCM_COEFF_ABC[14]*epsilon^4
%...


%FCM_COEFF_GH has the coefficients for the FCM terms for XG, YG, YH (60
%entires for each lambda)
%FCM_COEFF_M has the coefficients for the FCM terms for XM, YM (40 entries
%for each lambda)



function [] = fcm_coeff()

N = 2; %N is the number of particles.
lambda = [0.9];
fileID = fopen('output.txt','w');
str2 = 'double FCM_COEFF_ABC[] = {';
str3 = 'double FCM_COEFF_GH[] = {';
str4 = 'double FCM_COEFF_M[] = {';

format long

dist = .02:.01:1;
ngdh = 20;


for lval = lambda
    RA1 = zeros(1, length(dist));
    RA2 = zeros(1, length(dist));
    RA3 = zeros(1, length(dist));
    RA4 = zeros(1, length(dist));
    RA12 = zeros(1, length(dist));
    RA22 = zeros(1, length(dist));
    RA32 = zeros(1, length(dist));
    RA42 = zeros(1, length(dist));
    
    RB1 = zeros(1, length(dist));
    RB2 = zeros(1, length(dist));
    RC1 = zeros(1, length(dist));
    RC2 = zeros(1, length(dist));
    RC3 = zeros(1, length(dist));
    RC4 = zeros(1, length(dist));
    
    RB12 = zeros(1, length(dist));
    RB22 = zeros(1, length(dist));
    RC12 = zeros(1, length(dist));
    RC22 = zeros(1, length(dist));
    RC32 = zeros(1, length(dist));
    RC42 = zeros(1, length(dist));
    
    
    RG1 = zeros(1, length(dist));
    RG2 = zeros(1, length(dist));
    RG3 = zeros(1, length(dist));
    RG4 = zeros(1, length(dist));
    RG12 = zeros(1, length(dist));
    RG22 = zeros(1, length(dist));
    RG32 = zeros(1, length(dist));
    RG42 = zeros(1, length(dist));
    
    
    RH1 = zeros(1, length(dist));
    RH2 = zeros(1, length(dist));
    RH12 = zeros(1, length(dist));
    RH22 = zeros(1, length(dist));
    
    
    RM1 = zeros(1, length(dist));
    RM2 = zeros(1, length(dist));
    RM3 = zeros(1, length(dist));
    RM4 = zeros(1, length(dist));
    RM12 = zeros(1, length(dist));
    RM22 = zeros(1, length(dist));
    RM32 = zeros(1, length(dist));
    RM42 = zeros(1, length(dist));
    
    
    
    
    
    A = [1.0, lval];
    
    pscale = (6.0*sqrt(pi))^(1.0/3.0);
    siga=1.0/sqrt(pi);
    psiga = 1.0/pscale;
    
    
    
    for kdist =  1:length(dist)
        kdist
        eps = dist(kdist);
        Y = [0, 0, 0; 0, A(1)+A(2)+eps, 0]; %Assume one particle at origin.
        M = zeros(11*N, 11*N);
        
        
        n = 1;
        siga=A(n)/sqrt(pi);
        sigasq = siga*siga;
        anorm = 1.0/(pow(pi*2, 1.5)*pow(siga, 3.0));
        psiga = A(n)/pscale;
        psigasq = psiga*psiga;
        panorm = 1.0/(pow(pi*2, 1.5)*pow(psiga, 3.0));
        dx = 0.7*psiga;
        
        for kk = -ngdh:ngdh
            k =  kk;
            dzp = dx*k;
            gauss3 = exp(-0.5*dzp*dzp/sigasq);
            pgauss3 = exp(-0.5*dzp*dzp/psigasq);
            for jj = -ngdh:ngdh
                j = jj;
                dyp = dx*j;
                gauss2 = exp(-0.5*dyp*dyp/sigasq);
                pgauss2 = exp(-0.5*dyp*dyp/psigasq);
                for ii = -ngdh:ngdh
                    tempM1 = M(1:3, :);
                    M(1:3, :) = zeros(3, 11*N);
                    i =  ii;
                    dxp = dx*i;
                    gauss1 = exp(-0.5*dxp*dxp/sigasq);
                    pgauss1 = exp(-0.5*dxp*dxp/psigasq);
                    n2 = 1;
                    siga2=A(n2)/sqrt(pi);
                    psiga2=A(n2)/pscale;
                    x = dxp + Y(n, 1) - Y(n2, 1);
                    y = dyp + Y(n, 2) - Y(n2, 2);
                    z = dzp + Y(n, 3) - Y(n2, 3);
                    S = Oseen(x, y, z, siga2);
                    [R1a, R2a, R3a] = POseen(x, y, z, psiga2);
                    coeff = dx*dx*dx*anorm*gauss1*gauss2*gauss3;
                    pcoeff = dx*dx*dx*panorm*pgauss1*pgauss2*pgauss3/psigasq;
                    M(1:3, 1:3) = M(1:3, 1:3) + S;
                    
                    temp = [R1a(2,3) - R1a(3, 2), R1a(1, 3) - R1a(3, 1), R1a(1, 2) - R1a(2, 1); ...
                        R2a(2,3) - R2a(3, 2), R2a(1, 3) - R2a(3, 1), R2a(1, 2) - R2a(2, 1); ...
                        R3a(2,3) - R3a(3, 2), R3a(1, 3) - R3a(3, 1), R3a(1, 2) - R3a(2, 1)];
                    temp = 0.5*temp;
                    M(1:3, 7:9) = M(1:3, 7:9) + temp;
                    
                    temp = [(R1a(1, 1) -R1a(3, 3)), (R1a(2, 2)-R1a(3,3)), (R1a(2, 1)+R1a(1,2)), (R1a(1, 3) + R1a(3,1)), (R1a(2,3)+R1a(3,2)); ...
                        (R2a(1, 1) -R2a(3, 3)), (R2a(2, 2)-R2a(3,3)), (R2a(2, 1)+R2a(1,2)), (R2a(1, 3) + R2a(3,1)), (R2a(2,3)+R2a(3,2)); ...
                        (R3a(1, 1) -R3a(3, 3)), (R3a(2, 2)-R3a(3,3)), (R3a(2, 1)+R3a(1,2)), (R3a(1, 3) + R3a(3,1)), (R3a(2,3)+R3a(3,2))];
                    M(1:3, 13:17) = M(1:3, 13:17) + temp;
                    
                    n2 = 2;
                    siga2=A(n2)/sqrt(pi);
                    psiga2=A(n2)/pscale;
                    x = dxp + Y(n, 1) - Y(n2, 1);
                    y = dyp + Y(n, 2) - Y(n2, 2);
                    z = dzp + Y(n, 3) - Y(n2, 3);
                    S = Oseen(x, y, z, siga2);
                    [R1a, R2a, R3a] = POseen(x, y, z, psiga2);
                    coeff = dx*dx*dx*anorm*gauss1*gauss2*gauss3;
                    pcoeff = dx*dx*dx*panorm*pgauss1*pgauss2*pgauss3/psigasq;
                    M(1:3, 4:6) = M(1:3, 4:6) + S;
                    
                    temp = [R1a(2,3) - R1a(3, 2), R1a(1, 3) - R1a(3, 1), R1a(1, 2) - R1a(2, 1); ...
                        R2a(2,3) - R2a(3, 2), R2a(1, 3) - R2a(3, 1), R2a(1, 2) - R2a(2, 1); ...
                        R3a(2,3) - R3a(3, 2), R3a(1, 3) - R3a(3, 1), R3a(1, 2) - R3a(2, 1)];
                    temp = 0.5*temp;
                    M(1:3, 10:12) = M(1:3, 10:12) + temp;
                    temp = [(R1a(1, 1) -R1a(3, 3)), (R1a(2, 2)-R1a(3,3)), (R1a(2, 1)+R1a(1,2)), (R1a(1, 3) + R1a(3,1)), (R1a(2,3)+R1a(3,2)); ...
                        (R2a(1, 1) -R2a(3, 3)), (R2a(2, 2)-R2a(3,3)), (R2a(2, 1)+R2a(1,2)), (R2a(1, 3) + R2a(3,1)), (R2a(2,3)+R2a(3,2)); ...
                        (R3a(1, 1) -R3a(3, 3)), (R3a(2, 2)-R3a(3,3)), (R3a(2, 1)+R3a(1,2)), (R3a(1, 3) + R3a(3,1)), (R3a(2,3)+R3a(3,2))];
                    
                    M(1:3, 18:22) = M(1:3, 18:22) + temp;
                                        
                    M(7, :) = M(7, :) + 0.5*(M(3, :)*dyp - M(2,:)*dzp)*pcoeff;
                    M(8, :) = M(8, :) + 0.5*(M(1, :)*dzp - M(3,:)*dxp)*pcoeff;
                    M(9, :) = M(9, :) + 0.5*(M(2, :)*dxp - M(1,:)*dyp)*pcoeff;
                    M(13, :) = M(13, :) + dxp*pcoeff*M(1, :);
                    M(14, :) = M(14, :) + dyp*pcoeff*M(2, :);
                    M(15, :) = M(15, :) + 0.5*pcoeff*(dyp*M(1, :)+dxp*M(2, :));
                    M(16, :) = M(16, :) + 0.5*pcoeff*(dzp*M(1, :)+dxp*M(3, :));
                    M(17, :) = M(17, :) + 0.5*pcoeff*(dyp*M(3, :)+dzp*M(2, :));
                    M(1:3, :) = coeff*M(1:3, :)+tempM1;
                    
                end
            end
        end
        
        n = 2;
        siga=A(n)/sqrt(pi);
        sigasq = siga*siga;
        anorm = 1.0/(pow(pi*2, 1.5)*pow(siga, 3.0));
        psiga = A(n)/pscale;
        psigasq = psiga*psiga;
        panorm = 1.0/(pow(pi*2, 1.5)*pow(psiga, 3.0));
        dx = 0.7*psiga;
        
        for kk = -ngdh:ngdh
            k =  kk;
            dzp = dx*k;
            gauss3 = exp(-0.5*dzp*dzp/sigasq);
            pgauss3 = exp(-0.5*dzp*dzp/psigasq);
            for jj = -ngdh:ngdh
                j = jj;
                dyp = dx*j;
                gauss2 = exp(-0.5*dyp*dyp/sigasq);
                pgauss2 = exp(-0.5*dyp*dyp/psigasq);
                for ii = -ngdh:ngdh
                    
                    tempM2 = M(4:6, :);
                    M(4:6, :) = zeros(3, 11*N);
                    
                    i =  ii;
                    dxp = dx*i;
                    gauss1 = exp(-0.5*dxp*dxp/sigasq);
                    pgauss1 = exp(-0.5*dxp*dxp/psigasq);
                    n2 = 1;
                    siga2=A(n2)/sqrt(pi);
                    psiga2=A(n2)/pscale;
                    x = dxp + Y(n, 1) - Y(n2, 1);
                    y = dyp + Y(n, 2) - Y(n2, 2);
                    z = dzp + Y(n, 3) - Y(n2, 3);
                    S = Oseen(x, y, z, siga2);
                    [R1a, R2a, R3a] = POseen(x, y, z, psiga2);
                    coeff = dx*dx*dx*anorm*gauss1*gauss2*gauss3;
                    pcoeff = dx*dx*dx*panorm*pgauss1*pgauss2*pgauss3/psigasq;
                    M(4:6, 1:3) = M(4:6, 1:3) + S;
                    
                    temp = [R1a(2,3) - R1a(3, 2), R1a(1, 3) - R1a(3, 1), R1a(1, 2) - R1a(2, 1); ...
                        R2a(2,3) - R2a(3, 2), R2a(1, 3) - R2a(3, 1), R2a(1, 2) - R2a(2, 1); ...
                        R3a(2,3) - R3a(3, 2), R3a(1, 3) - R3a(3, 1), R3a(1, 2) - R3a(2, 1)];
                    temp = 0.5*temp;
                    M(4:6, 7:9) = M(4:6, 7:9) + temp;
                    
                    temp = [(R1a(1, 1) -R1a(3, 3)), (R1a(2, 2)-R1a(3,3)), (R1a(2, 1)+R1a(1,2)), (R1a(1, 3) + R1a(3,1)), (R1a(2,3)+R1a(3,2)); ...
                        (R2a(1, 1) -R2a(3, 3)), (R2a(2, 2)-R2a(3,3)), (R2a(2, 1)+R2a(1,2)), (R2a(1, 3) + R2a(3,1)), (R2a(2,3)+R2a(3,2)); ...
                        (R3a(1, 1) -R3a(3, 3)), (R3a(2, 2)-R3a(3,3)), (R3a(2, 1)+R3a(1,2)), (R3a(1, 3) + R3a(3,1)), (R3a(2,3)+R3a(3,2))];
                    M(4:6, 13:17) = M(4:6, 13:17) + temp;

                    n2 = 2;
                    siga2=A(n2)/sqrt(pi);
                    psiga2=A(n2)/pscale;
                    x = dxp + Y(n, 1) - Y(n2, 1);
                    y = dyp + Y(n, 2) - Y(n2, 2);
                    z = dzp + Y(n, 3) - Y(n2, 3);
                    S = Oseen(x, y, z, siga2);
                    [R1a, R2a, R3a] = POseen(x, y, z, psiga2);
                    coeff = dx*dx*dx*anorm*gauss1*gauss2*gauss3;
                    pcoeff = dx*dx*dx*panorm*pgauss1*pgauss2*pgauss3/psigasq;
                    M(4:6, 4:6) = M(4:6, 4:6) + S;
                    temp = [R1a(2,3) - R1a(3, 2), R1a(1, 3) - R1a(3, 1), R1a(1, 2) - R1a(2, 1); ...
                        R2a(2,3) - R2a(3, 2), R2a(1, 3) - R2a(3, 1), R2a(1, 2) - R2a(2, 1); ...
                        R3a(2,3) - R3a(3, 2), R3a(1, 3) - R3a(3, 1), R3a(1, 2) - R3a(2, 1)];
                    temp = 0.5*temp;
                    M(4:6, 10:12) = M(4:6, 10:12) + temp;
                    temp = [(R1a(1, 1) -R1a(3, 3)), (R1a(2, 2)-R1a(3,3)), (R1a(2, 1)+R1a(1,2)), (R1a(1, 3) + R1a(3,1)), (R1a(2,3)+R1a(3,2)); ...
                        (R2a(1, 1) -R2a(3, 3)), (R2a(2, 2)-R2a(3,3)), (R2a(2, 1)+R2a(1,2)), (R2a(1, 3) + R2a(3,1)), (R2a(2,3)+R2a(3,2)); ...
                        (R3a(1, 1) -R3a(3, 3)), (R3a(2, 2)-R3a(3,3)), (R3a(2, 1)+R3a(1,2)), (R3a(1, 3) + R3a(3,1)), (R3a(2,3)+R3a(3,2))];
                    
                    M(4:6, 18:22) = M(4:6, 18:22) + temp;
                    
                    
                    M(10, :) = M(10, :) + 0.5*(M(6, :)*dyp - M(5,:)*dzp)*pcoeff;
                    M(11, :) = M(11, :) + 0.5*(M(4, :)*dzp - M(6,:)*dxp)*pcoeff;
                    M(12, :) = M(12, :) + 0.5*(M(5, :)*dxp - M(4,:)*dyp)*pcoeff;
                    
                    M(18, :) = M(18, :) + dxp*pcoeff*M(4, :);
                    M(19, :) = M(19, :) + dyp*pcoeff*M(5, :);
                    M(20, :) = M(20, :) + 0.5*pcoeff*(dyp*M(4, :)+dxp*M(5, :));
                    M(21, :) = M(21, :) + 0.5*pcoeff*(dzp*M(4, :)+dxp*M(6, :));
                    M(22, :) = M(22, :) + 0.5*pcoeff*(dyp*M(6, :)+dzp*M(5, :));
                    
                    M(4:6, :) = coeff*M(4:6, :)+tempM2;
                    
                    
                end
            end
        end

        Minv = M^(-1);
        RA1(1, kdist) = Minv(1, 1);
        RA2(1, kdist) = Minv(2, 2);
        RA3(1, kdist) = Minv(1, 4);
        RA4(1, kdist) = Minv(2, 5);
        
        RA12(1, kdist) = Minv(4, 4);
        RA22(1, kdist) = Minv(5, 5);
        RA32(1, kdist) = Minv(4, 1);
        RA42(1, kdist) = Minv(5, 2);

        RB1(1, kdist) = Minv(3, 7);
        RB2(1, kdist) = Minv(1, 12);
        RB12(1, kdist) = Minv(6, 10);
        RB22(1, kdist) = Minv(4, 9);
        
        RC1(1, kdist) = Minv(7, 7);
        RC2(1, kdist) = Minv(8, 8);
        RC3(1, kdist) = Minv(7, 10);
        RC4(1, kdist) = Minv(8, 11);
        
        RC12(1, kdist) = Minv(10, 10);
        RC22(1, kdist) = Minv(11, 11);
        RC32(1, kdist) = Minv(10, 7);
        RC42(1, kdist) = Minv(11, 8);
        
        RG1(1, kdist) = Minv(2, 14);
        RG2(1, kdist) = Minv(1, 15);
        RG3(1, kdist) = Minv(2, 19);
        RG4(1, kdist) = Minv(1, 20);
        
        RG12(1, kdist) = Minv(5, 19);
        RG22(1, kdist) = Minv(4, 20);
        RG32(1, kdist) = Minv(5, 14);
        RG42(1, kdist) = Minv(4, 15);
        
        RG1(1, kdist) = Minv(2, 14);
        RG2(1, kdist) = Minv(1, 15)/2;
        RG3(1, kdist) = Minv(2, 19);
        RG4(1, kdist) = Minv(1, 20)/2;
        
        RG12(1, kdist) = Minv(5, 19);
        RG22(1, kdist) = Minv(4, 20)/2;
        RG32(1, kdist) = Minv(5, 14);
        RG42(1, kdist) = Minv(4, 15)/2;

        RH1(1, kdist) = Minv(7, 17)/2;
        RH2(1, kdist) = Minv(7, 22)/2;
        RH12(1, kdist) = Minv(10, 22)/2;
        RH22(1, kdist) = Minv(10, 17)/2;
        
        RM1(1, kdist) = Minv(14, 14);
        RM2(1, kdist) = Minv(17, 17); 
        RM3(1, kdist) = Minv(14, 19);
        RM4(1, kdist) = Minv(17, 22); 
        
        RM12(1, kdist) = Minv(19, 19);
        RM22(1, kdist) = Minv(22, 22);
        RM32(1, kdist) = Minv(19, 14);
        RM42(1, kdist) = Minv(22, 17); 
    end

    eps = dist;
    
    xa1 = polyfit(dist, RA2,  4);
    xa2 = polyfit(dist, RA4,  4);
    ya1 = polyfit(dist, RA1,  4);
    ya2 = polyfit(dist, RA3,  4);
    
    xa1_2 = polyfit(dist, RA22,  4);
    xa2_2 = polyfit(dist, RA42,  4);
    ya1_2 = polyfit(dist, RA12,  4);
    ya2_2 = polyfit(dist, RA32,  4);

    yb1 = polyfit(dist, RB1, 4);
    yb2 = polyfit(dist, RB2, 4);
    yb1_2 = polyfit(dist, RB12, 4);
    yb2_2 = polyfit(dist, RB22, 4);
    
    xc1 = polyfit(dist, RC2,  4);
    xc2 = polyfit(dist, RC4,  4);
    yc1 = polyfit(dist, RC1,  4);
    yc2 = polyfit(dist, RC3,  4);
    xc1_2 = polyfit(dist, RC22,  4);
    xc2_2 = polyfit(dist, RC42,  4);
    yc1_2 = polyfit(dist, RC12,  4);
    yc2_2 = polyfit(dist, RC32,  4);
    
    xg1 = polyfit(dist, RG1,  4);
    xg2 = polyfit(dist, RG3,  4);
    yg1 = polyfit(dist, RG2,  4);
    yg2 = polyfit(dist, RG4,  4);
    
    xg1_2 = polyfit(dist, RG12,  4);
    xg2_2 = polyfit(dist, RG32,  4);
    yg1_2 = polyfit(dist, RG22,  4);
    yg2_2 = polyfit(dist, RG42,  4);
    
    yh1 = polyfit(dist, RH1, 4);
    yh2 = polyfit(dist, RH2, 4);
    yh1_2 = polyfit(dist, RH12, 4);
    yh2_2 = polyfit(dist, RH22, 4);

    xm1 = polyfit(dist, RM1,  4);
    xm2 = polyfit(dist, RM3,  4);
    ym1 = polyfit(dist, RM2,  4);
    ym2 = polyfit(dist, RM4,  4);
    
    xm1_2 = polyfit(dist, RM12,  4);
    xm2_2 = polyfit(dist, RM32,  4);
    ym1_2 = polyfit(dist, RM22,  4);
    ym2_2 = polyfit(dist, RM42,  4);

    a1 = 1.0;
    a2 = lval;
    
    xa1 = fliplr(xa1/3/pi/(a1 + a1));
    xa2 = fliplr(xa2/3/pi/(a1 + a2));
    xa1_2 = fliplr(xa1_2/3/pi/(a2 + a2));
    xa2_2 = fliplr(xa2_2/3/pi/(a2 + a1));
    
    ya1 = fliplr(ya1/3/pi/(a1 + a1));
    ya2 = fliplr(ya2/3/pi/(a1 + a2));
    ya1_2 = fliplr(ya1_2/3/pi/(a2 + a2));
    ya2_2 = fliplr(ya2_2/3/pi/(a2 + a1));
    
    yb1 = fliplr(yb1/1.5/pi/(a1+a1)^2);
    yb2 = fliplr(yb2/1.5/pi/(a1+a2)^2);
    yb1_2 = fliplr(yb1_2/1.5/pi/(a2+a2)^2);
    yb2_2 = fliplr(yb2_2/1.5/pi/(a1+a2)^2);
    
    xc1 = fliplr(xc1/pi/(a1+a1)^3);
    xc2 = fliplr(xc2/pi/(a2+a1)^3);
    xc1_2 = fliplr(xc1_2/pi/(a2+a2)^3);
    xc2_2 = fliplr(xc2_2/pi/(a1+a2)^3);
    
    yc1 = fliplr(yc1/pi/(a1+a1)^3);
    yc2 = fliplr(yc2/pi/(a2+a1)^3);
    yc1_2 = fliplr(yc1_2/pi/(a2+a2)^3);
    yc2_2 = fliplr(yc2_2/pi/(a1+a2)^3);
    
    retval = [xa1, xa2, xa1_2, xa2_2, ya1,  ya2, ya1_2, ya2_2, -yb1, ...
        -yb2, yb1_2, yb2_2, -xc1, -xc2, -xc1_2, -xc2_2, yc1, yc2, yc1_2, yc2_2];
    
    
    xg1 = fliplr(xg1/1.5/pi/(a1+a1)^2);
    xg2 = fliplr(xg2/1.5/pi/(a2+a1)^2);
    xg1_2 = fliplr(xg1_2/1.5/pi/(a2+a2)^2);
    xg2_2 = fliplr(xg2_2/1.5/pi/(a1+a2)^2);
    
    yg1 = fliplr(yg1/1.5/pi/(a1+a1)^2);
    yg2 = fliplr(yg2/1.5/pi/(a2+a1)^2);
    yg1_2 = fliplr(yg1_2/1.5/pi/(a2+a2)^2);
    yg2_2 = fliplr(yg2_2/1.5/pi/(a1+a2)^2);
    
    yh1 = fliplr(yh1/pi/(a1+a1)^3);
    yh2 = fliplr(yh2/pi/(a1+a2)^3);
    yh1_2 = fliplr(yh1_2/pi/(a2+a2)^3);
    yh2_2 = fliplr(yh2_2/pi/(a1+a2)^3);
    
    retval2 = [xg1, -xg2, -xg1_2, xg2_2, yg1, -yg2, -yg1_2, yg2_2, yh1, yh2, yh1_2, yh2_2];
    
    scale = 5/6*pi;
    xm1 = fliplr(xm1/scale/(a1+a1)^3);
    xm2 = fliplr(xm2/scale/(a2+a1)^3);
    xm1_2 = fliplr(xm1_2/scale/(a2+a2)^3);
    xm2_2 = fliplr(xm2_2/scale/(a1+a2)^3);
    
    ym1 = fliplr(ym1/scale/(a1+a1)^3);
    ym2 = fliplr(ym2/scale/(a2+a1)^3);
    ym1_2 = fliplr(ym1_2/scale/(a2+a2)^3);
    ym2_2 = fliplr(ym2_2/scale/(a1+a2)^3);

    retval3 = [xm1, xm2, xm1_2, xm2_2, ym1, ym2, ym1_2, ym2_2] ;
    
    for i = 1:length(retval)
        str2 = [str2, num2str(retval(i)), ', '];
        if mod(i, 10) == 0
            str2 = [str2, char(10)];
        end
        if mod(i, 100) == 0
            str2 = [str2, char(10)];
        end
        
    end
    for i = 1:length(retval2)
        str3 = [str3, num2str(retval2(i)), ', '];
        if mod(i, 10) == 0
            str3 = [str3, char(10)];
        end
        if mod(i, 60) == 0
            str3 = [str3, char(10)];
        end
    end
    for i = 1:length(retval3)
        str4 = [str4, num2str(retval3(i)), ', '];
        if mod(i, 10) == 0
            str4 = [str4, char(10)];
        end
        if mod(i, 40) == 0
            str4 = [str4, char(10)];
        end
    end
    
end

str2 = str2(1:end-4);
str2 = [str2 '};'];
str3 = str3(1:end-4);
str3 = [str3 '};'];
str4 = str4(1:end-4);
str4 = [str4 '};'];

fprintf(fileID,'%s\n',str2);
fprintf(fileID,'%s\n',str3);
fprintf(fileID,'%s\n',str4);

fclose(fileID);
end

function a = pow(x, n)
a = x^n;
end

function [S] = Oseen( x,  y,  z, sigma)
sigsq = sigma*sigma;
anorm = 1.0/sqrt(pi*2);
rsq = x*x + y*y + z*z;
r = sqrt(rsq);
X = zeros(1, 3);
X(1, 1) = x;
X(1, 2) = y;
X(1, 3) = z;
if abs(r/sigma) < 0.01
    af = anorm/(3.0 * pi *sigma);
    bf = af/(10.0 * sigsq);
else
    delta = anorm*exp(-0.5*rsq/sigsq);
    gerf = erf(r / (sigma*sqrt(2.0)));
    atemp = (1.0+sigsq/rsq)*gerf - (2.0*sigma/r)*delta;
    btemp = (1.0-3.0*sigsq/rsq)*gerf + (6.0*sigma/r)*delta;
    af = atemp/(8.0*pi*r);
    bf = btemp/(8.0*pi*r*rsq);
end
S = zeros(3, 3);
for j = 1:3
    for i = 1:3
        S(i, j) = bf*X(1, i)*X(1, j);
    end
    S(j, j) = af + S(j, j);
end
end

function [R1, R2, R3] = POseen( x,  y,  z, sigma)
sigsq = sigma * sigma;
anorm = 1.0/sqrt(pi*2);
rsq = x*x + y*y + z*z;
r = sqrt(rsq);
X = zeros(1, 3);
X(1,1) = x;
X(1,2) = y;
X(1,3) = z;
R1 = zeros(3,3);
R2 = zeros(3,3);
R3 = zeros(3,3);
if abs(r/sigma) < 0.01
    af = anorm/(pi * sigma);
    dadrf = -2.0*af/(15.0*sigsq);
    dbdrf = -32.0*af/(35.0*sigsq*sigsq);
    bf = af/(30.0*sigsq);
    for j = 1:3
        for i = 1:3
            R1(j, i) = R1(j, i) + dbdrf*X(1,1)*X(1, i)*X(1, j);
            R2(j, i) = R2(j, i) + dbdrf*X(1,2)*X(1, i)*X(1, j);
            R3(j, i) = R3(j, i) + dbdrf*X(1,3)*X(1, i)*X(1, j);
            if j == 1
                R1(j, i) = R1(j, i) + dadrf*X(1, i);
            end
            if j == 2
                R2(j, i) = R2(j, i) + dadrf*X(1, i);
            end
            if j == 3
                R3(j, i) = R3(j, i) + dadrf*X(1, i);
            end
            if i == 1
                R1(j, i) = R1(j, i) + bf*X(1, j);
            end
            if i == 2
                R2(j, i) = R2(j, i) + bf*X(1, j);
            end
            if i == 3
                R3(j, i) = R3(j, i) + bf*X(1, j);
            end
        end
        R1(j, j) = R1(j, j) + bf*X(1, 1);
        R2(j, j) = R2(j, j) + bf*X(1, 2);
        R3(j, j) = R3(j, j) + bf*X(1, 3);
    end
else
    delta = anorm * exp(-0.5*rsq/sigsq);
    gerf = erf(r/(sigma*sqrt(2)));
    dadrtemp = (1.0+3.0*sigsq/rsq)*gerf - (4.0*r/sigma + 6.0*sigma/r)*delta;
    btemp = (1.0 - 3.0*sigsq/rsq)*gerf+(6.0*sigma/r)*delta;
    dbdrtemp = (3.0 - 15.0*sigsq/rsq)*gerf + (4.0*r/sigma + 30.0*sigma/r)*delta;
    dadrf = -dadrtemp/(8.0*pi*rsq);
    bf = btemp/(8.0*pi*r*rsq);
    dbdrf = -dbdrtemp/(8.0*pi*rsq*rsq);
    for j = 1:3
        for i = 1:3
            R1(j, i) = R1(j, i)+ dbdrf*X(1,1)*X(1, i)*X(1, j)/r;
            R2(j, i) = R2(j, i)+ dbdrf*X(1,2)*X(1, i)*X(1, j)/r;
            R3(j, i) = R3(j, i)+ dbdrf*X(1,3)*X(1, i)*X(1, j)/r;
            if j == 1
                R1(j, i) = R1(j, i) + dadrf*X(1, i)/r;
            end
            if (j == 2)
                R2(j, i) = R2(j, i) + dadrf*X(1, i)/r;
            end
            if (j == 3)
                R3(j, i) = R3(j, i) + dadrf*X(1, i)/r;
            end
            if (i == 1)
                R1(j, i) = R1(j, i) + bf*X(1, j);
            end
            if (i == 2)
                R2(j, i) = R2(j, i) + bf*X(1, j);
            end
            if (i == 3)
                R3(j, i) = R3(j, i) + bf*X(1, j);
            end
        end
        R1(j, j) = R1(j, j) + bf*X(1, 1);
        R2(j, j) = R2(j, j) + bf*X(1, 2);
        R3(j, j) = R3(j, j) + bf*X(1, 3);
    end
end
end

